#include "ShaderConstants.fxh"
#include "util.fxh"
#include "functions.hlsl"

struct PS_Input
{
	float4 position : SV_Position;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
	float3 wpos : worldPosition;
	float3 vpos : vpos;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{
#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else

#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0 );
	float4 diffl = texture2D_AA(TEXTURE_1, TextureSampler1, float2(0.0, 1.0));
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
	float4 diffl = TEXTURE_1.Sample(TextureSampler1, float2(0.0, 1.0));
#endif

#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

#if defined(BLEND)
	diffuse.a *= PSInput.color.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse = diffuse * TEXTURE_1.Sample(TextureSampler1, PSInput.uv1);
#endif
    float rain = 1.0-pow(FOG_CONTROL.y, 11.0);
#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = PSInput.color.a;
	#endif	

	diffuse.rgb *= PSInput.color.rgb;
#else
	float2 uv = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif

#ifdef BLEND
diffuse += pow(max(0.0,dot(reflect(-(normalize(float3(10000,7000,200)-PSInput.vpos)),normalize(PSInput.wpos)),normalize(-PSInput.wpos))),70.0)*(float4(1.2,1.2,1.2,1.0)*0.7);  
if(PSInput.color.b>PSInput.color.r){
    diffuse.rgb = lerp(diffuse.rgb,FOG_COLOR.rgb*1.5,clamp(length(PSInput.wpos)/RENDER_DISTANCE*1.0,0.0,2.0));
	}
#endif
    diffuse.rgb *= lerp(lerp(0.5,1.0,smoothstep(0.76,0.85,PSInput.uv1.y)),1.0,PSInput.uv1.x);
    diffuse.rgb = lerp(diffuse.rgb,FOG_COLOR.rgb*1.2,clamp(length(-PSInput.wpos.xz *2.)/FAR_CHUNKS_DISTANCE*.2,.1,.8));
#ifdef FOG
//	diffuse.rgb = lerp(diffuse.rgb,PSInput.fogColor.rgb,PSInput.fogColor.a);
#endif
    float3 fogg = lerp(diffuse.rgb,FOG_COLOR.rgb,clamp(length(-PSInput.wpos*0.5)/RENDER_DISTANCE*1.0,0.0,1.0));
    diffuse.rgb = fogg; 
	diffuse.rgb = saturation(diffuse.rgb,1.28);
	
	diffuse.rgb = clamp((2/1.3)*(diffuse.rgb-0.5)+0.6,0,1);
	diffuse.rgb = pow(diffuse.rgb,1.3);
	diffuse.rgb = clamp(diffuse.rgb,0,1);

	PSOutput.color = diffuse;
#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}